/****************************************************************************\
**  Title:       PCXHEAD.C                                                  **
**  Purpose:     Display the data in a PCX image file header.               **
**  Version:     1.0                                                        **
**  Date:        April 1991                                                 **
**  Author:      James D. Murray, Anaheim, CA, USA                          **
**  C Compilers: Borland C++ v2.0, Microsoft C v6.00a                       **
**                                                                          **
**  PCXHEAD displays all real information contained within the header of a  **
**  PCX image file.  The EGA color palette (map) is always displayed        **
**  because this color palette always exists in the PCX header even if it   **
**  if it is all zeros.  If a VGA color palette exists then it is also      **
**  displayed.                                                              **
**                                                                          **
**  Copyright (C) 1991 by Graphics Software Labs.  All rights reserved.     **
\****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <io.h>
#include "endianio.h"
#include "pcx.h"

#define VERSION                     "1.00"


int
main(argc, argv)
int argc;           /* Number of command line arguments */
char *argv[];       /* Array of command line arguments  */
{
    WORD           i, j;            /* Loop Counters                        */
    LONG           imageSize;       /* Size of the PCX image file           */
    CHAR           pcxFileName[81]; /* Holder for the PCX image file name   */
    FILE          *fpPcx;           /* File pointer to the PCX image file   */
    PCXHEADER      pcxHead;         /* PCX image file header structure      */
    PCXVGAPALETTE  pcxVgaPal;       /* PCX image file VGA palette structure */

    printf("PCXHEAD - Display the header info within a PCX image file (v%s)\n",
      VERSION);

    /* Check for proper number of command line arguments */
    if (argc < 2)
    {
        fputs("Usage: PCXHEAD filename.pcx\n\n", stderr);
        exit(-1);
    }

    /* Add the .pcx extension to the file name if no extension exists */
    strncpy(pcxFileName, argv[1], 76);
    if (!strrchr(pcxFileName, '.'))
        strcat(pcxFileName, ".pcx");

    /* Open the PCX image file */
    if ((fpPcx = fopen(pcxFileName, "rb")) == (FILE *) NULL)
    {
        fprintf(stderr, "PCXHEAD: Cannot open file %s\n", pcxFileName);
        exit(-2);
    }

    /* Read the PCX image file header information */
    ReadPcxHeader(&pcxHead, fpPcx);

    /* Check for FILE stream error */
    if (ferror(fpPcx))
    {
        fputs("PCXHEAD: Error reading header information!\n", stderr);
        exit(-3);
    }

    /* Check the identification byte value */
    if (pcxHead.Id != 0x0A)
    {
        fprintf(stderr, "PCXHEAD: %s is not a PCX-format file!\n",
          pcxFileName);
        exit(-4);
    }

    /* Get the size of the image file */
    imageSize = filelength(fileno(fpPcx));

    /*
    ** Display, in an orderly fashion, all the information now contained
    ** within the PCXHEADER structure.  The following code looks like
    ** hell in an attempt to make the information look nice on screen.
    **
    ** Note the Horizontal and Vertical Screen Size field values are not
    ** displayed as they are only present in headers of PCX files created
    ** by PC Paintbrush, PC Paintbrush Plus, PC Paintbrush for Windows,
    ** and compatable programs (Version field values 4 or 5) and are
    ** typically not used.
    */
    puts("                                            EGA Color Palette");
    puts("                                          Color  Red  Green  Blue");
    printf("\n             PCX File: %-16s", pcxFileName);
    printf("     0    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[0], pcxHead.EgaPalette[1], pcxHead.EgaPalette[2]);
    
    printf("   Size of Image File: %-16ld", imageSize);
    printf("     1    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[3], pcxHead.EgaPalette[4], pcxHead.EgaPalette[5]);
    
    printf("              Version: ");
    switch((int) pcxHead.Version) {
        case 0:
            printf("%-16s", "2.5");
            break;
        case 1:
            printf("%-16s", "2.8");
            break;
        case 3:
            printf("%-16s", "2.8 (no palette)");
            break;
        case 5:
            printf("%-16s", "3.0");
            break;
        default:
            printf("%-16s", "Unknown");
            break;
    }
    printf("     2    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[6], pcxHead.EgaPalette[7], pcxHead.EgaPalette[8]);
    
    printf("             Encoding: %-16s", pcxHead.Format == 1 ?
      "RLE" : (pcxHead.Format == 0 ? "None" : "Unknown"));
    printf("     3    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[9], pcxHead.EgaPalette[10], pcxHead.EgaPalette[11]);

    printf("       Bits per Pixel: %-16d", pcxHead.BitsPixelPlane);
    printf("     4    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[12], pcxHead.EgaPalette[13], pcxHead.EgaPalette[14]);
    
    printf("              X Start: %-16d", pcxHead.Xmin);
    printf("     5    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[15], pcxHead.EgaPalette[16], pcxHead.EgaPalette[17]);
    
    printf("              Y Start: %-16d", pcxHead.Ymin);
    printf("     6    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[18], pcxHead.EgaPalette[19], pcxHead.EgaPalette[20]);
    
    printf("                X End: %-16d", pcxHead.Xmax);
    printf("     7    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[21], pcxHead.EgaPalette[22], pcxHead.EgaPalette[23]);
    
    printf("                Y End: %-16d", pcxHead.Ymax);
    printf("     8    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[24], pcxHead.EgaPalette[25], pcxHead.EgaPalette[26]);
    
    printf("Horizontal Resolution: %-16d", pcxHead.Hdpi);
    printf("     9    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[27], pcxHead.EgaPalette[28], pcxHead.EgaPalette[29]);
    
    printf("  Vertical Resolution: %-16d", pcxHead.Vdpi);
    printf("    10    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[30], pcxHead.EgaPalette[31], pcxHead.EgaPalette[32]);
    
    printf("        Reserved Byte: %-16d", pcxHead.Reserved);
    printf("    11    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[33], pcxHead.EgaPalette[34], pcxHead.EgaPalette[35]);
    
    printf(" Number of Bit Planes: %-16d", pcxHead.NumberOfPlanes);
    printf("    12    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[36], pcxHead.EgaPalette[37], pcxHead.EgaPalette[38]);
    
    printf("  Bytes per Scan Line: %-16d", pcxHead.BytesLinePlane * pcxHead.Numb
erOfPlanes);
    printf("    13    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[39], pcxHead.EgaPalette[40], pcxHead.EgaPalette[41]);
    
    printf("         Palette Type: %-16s", (pcxHead.PaletteInfo == 1 ?
      "Color or B&W" : (pcxHead.PaletteInfo == 2 ? "Gray Scale" :
      "Unknown")));
    printf("    14    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[42], pcxHead.EgaPalette[43], pcxHead.EgaPalette[44]);
                 
    printf(" Max Number of Colors: %-16ld", 1L << (pcxHead.NumberOfPlanes * pcxH
ead.BitsPixelPlane));
    printf("    15    %03d   %03d   %03d\n",
      pcxHead.EgaPalette[45], pcxHead.EgaPalette[46], pcxHead.EgaPalette[47]);

    /* Check for the presence of a VGA color map */
    if (pcxHead.Version == 5)
    {
        fseek(fpPcx, -769L, SEEK_END);  /* Seek to the VGA Id byte */

        if (GetByte(fpPcx) == 0x0C)     /* Is it the proper value? */
        {
            fputs("\nHit Enter for VGA palette information...", stdout);
            getch();

            /* Read VGA palette information from PCX file */
            for (i = 0; i < sizeof(pcxVgaPal); i++)
                pcxVgaPal.VgaPalette[i] = GetByte(fpPcx);

            if (ferror(fpPcx))
                fputs("PCXHEAD: Error reading VGA palette information!\n", stder
r);
            else
            {
                puts("\n\n                                VGA Color Palette\n");

                puts("Color  Red Grn Blu  Color  Red Grn Blu  Color  Red Grn Blu
  Color  Red Grn Blu");

                /* Display the palette values */
                for (i = 0, j = 0; i < 256; i += 4, j += 12)
                {
                    printf(" %3d   %03u %03u %03u   %3d   %03u %03u %03u",
                      i,
                      pcxVgaPal.VgaPalette[j],
                      pcxVgaPal.VgaPalette[j+1],
                      pcxVgaPal.VgaPalette[j+2],
                      i+1,
                      pcxVgaPal.VgaPalette[j+3],
                      pcxVgaPal.VgaPalette[j+4],
                      pcxVgaPal.VgaPalette[j+5]);
                      
                    printf("   %3d   %03u %03u %03u   %3d   %03u %03u %03u\n",
                      i+2,
                      pcxVgaPal.VgaPalette[j+6],
                      pcxVgaPal.VgaPalette[j+7],
                      pcxVgaPal.VgaPalette[j+8],
                      i+3,
                      pcxVgaPal.VgaPalette[j+9],
                      pcxVgaPal.VgaPalette[j+10],
                      pcxVgaPal.VgaPalette[j+11]);

                    if (i && i % 88 == 0)
                    {
                        fputs("Hit Enter for next page...", stdout);
                        getch();        
                        puts("\n\nColor  Red Grn Blu  Color  Red Grn Blu  \
                              Color  Red Grn Blu  Color  Red Grn Blu");
                    }
                }
                putchar('\n');
            }
        }
        else
            puts("\nPCXHEAD: No VGA color palette found.");
    }
    fclose(fpPcx);

    return(0);              /* Successful termination */
}
}
